package com.mt.wms.empty.controller;

import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.http.HttpResponse;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSON;
import cn.hutool.json.JSONUtil;
import com.mt.wms.core.base.BaseController;
import com.mt.wms.core.constants.CommonConstant;
import com.mt.wms.core.dal.entity.CommunicationLog;
import com.mt.wms.core.dal.service.CommunicationLogServiceBiz;
import com.mt.wms.core.vo.R;
import com.mt.wms.empty.params.*;
import com.mt.wms.empty.service.OrderInfoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDateTime;
import java.util.HashMap;

/**
 * @author xcc
 * @date 2021年11月18日
 * @since 1.0
 */
@RestController
@RequestMapping(CommonConstant.API_MODULE_BASE + "apms")
@Slf4j
@Api(value = "Apms相关接口", tags = "Apms相关接口", hidden = false)
public class ApmsController extends BaseController {
    @Autowired
    private OrderInfoService orderInfoService;

    @Autowired
    private CommunicationLogServiceBiz communicationLogServiceBiz;

    static String Token = "";
    static LocalDateTime TokenExpireTime = LocalDateTime.now();
    static String ApiAddress = "http://59.110.171.25:9010";

    @PostMapping(value = "apmsPostOrder")
    @ApiOperation(value = "接收APMS推送订单信息")
    public R apmsPostOrder(@Validated @RequestBody OrderParamForApms apmsPostOrderParam) {
        CommunicationLog communicationLog = new CommunicationLog();
        communicationLog.setCode("APMS" + System.currentTimeMillis());
        communicationLog.setLogName("WMS接收APMS订单");
        communicationLog.setContent(apmsPostOrderParam.toString());
        communicationLog.setType(3);
        setCommonField(communicationLog);
        communicationLogServiceBiz.save(communicationLog);
        return orderInfoService.apmsPostOrder(apmsPostOrderParam);
    }

    @PostMapping(value = "apmsVoidOrder")
    @ApiOperation(value = "APMS作废订单")
    public R apmsVoidOrder(@Validated @RequestBody String idenCardNum) {
        CommunicationLog communicationLog = new CommunicationLog();
        communicationLog.setCode("APMS" + System.currentTimeMillis());
        communicationLog.setLogName("APMS作废订单");
        communicationLog.setType(3);
        communicationLog.setContent(idenCardNum);
        setCommonField(communicationLog);
        communicationLogServiceBiz.save(communicationLog);
        return orderInfoService.apmsVoidOrder(idenCardNum);
    }

    @PostMapping(value = "apmsCompleteOrder")
    @ApiOperation(value = "APMS完成订单,返回加工单检验结果")
    public R apmsCompleteOrder(@Validated @RequestBody ApmsCompleteOrder apmsCompleteOrder) {
        CommunicationLog communicationLog = new CommunicationLog();
        communicationLog.setCode("APMS" + System.currentTimeMillis());
        communicationLog.setLogName("APMS完成订单,返回WMS加工单检验结果");
        communicationLog.setType(3);
        communicationLog.setContent(apmsCompleteOrder.toString());
        setCommonField(communicationLog);
        communicationLogServiceBiz.save(communicationLog);
        //APMS完成订单,返回检验结果，更新结果和后续处理
        return orderInfoService.apmsCompleteOrder(apmsCompleteOrder);
    }

    @PostMapping(value = "createProcessSheet")
    @ApiOperation(value = "通知APMS创建生产单接口")
    public R createProcessSheet(@Validated @RequestBody ApmsCreateProcessSheet apmsCreateProcess) {
        checkToken();
        HashMap<String, Object> paramMap = new HashMap<>();
        paramMap.put("entity", apmsCreateProcess);
        HttpResponse response = HttpUtil.createPost(ApiAddress + "/platform/api/createProcessSheet")
                .header("token", Token).form(paramMap).execute();
        CommunicationLog communicationLog = new CommunicationLog();
        communicationLog.setCode("APMS" + System.currentTimeMillis());
        communicationLog.setLogName("WMS通知APMS创建生产单");
        communicationLog.setType(2);
        communicationLog.setContent("param:" + paramMap + "\nresult:" + response.body());
        setCommonField(communicationLog);
        communicationLogServiceBiz.save(communicationLog);
        return successful(response);
    }

    @PostMapping(value = "startProcess")
    @ApiOperation(value = "通知APMS生产单开始处理")
    public R startProcess(@Validated @RequestBody ApmsStartProcess apmsStartProcess) {
        checkToken();
        HashMap<String, Object> paramMap = new HashMap<>();
        paramMap.put("entity", apmsStartProcess);
        HttpResponse response = HttpUtil.createPost(ApiAddress + "/platform/api/startProcess")
                .header("token", Token).form(paramMap).execute();
        CommunicationLog communicationLog = new CommunicationLog();
        communicationLog.setCode("APMS" + System.currentTimeMillis());
        communicationLog.setLogName("WMS通知APMS生产单开始处理");
        communicationLog.setType(2);
        communicationLog.setContent("param:" + paramMap + "\nresult:" + response.body());
        setCommonField(communicationLog);
        communicationLogServiceBiz.save(communicationLog);
        return successful(response);
    }

    @PostMapping(value = "endProcess")
    @ApiOperation(value = "通知APMS生产单结束处理")
    public R endProcess(@Validated @RequestBody ApmsEndProcess apmsEndProcess) {
        checkToken();
        HashMap<String, Object> paramMap = new HashMap<>();
        paramMap.put("entity", apmsEndProcess);
        HttpResponse response = HttpUtil.createPost(ApiAddress + "/platform/api/endProcess")
                .header("token", Token).form(paramMap).execute();
        CommunicationLog communicationLog = new CommunicationLog();
        communicationLog.setCode("APMS" + System.currentTimeMillis());
        communicationLog.setLogName("WMS通知APMS生产单结束处理");
        communicationLog.setType(2);
        communicationLog.setContent("param:" + paramMap + "\nresult:" + response.body());
        setCommonField(communicationLog);
        communicationLogServiceBiz.save(communicationLog);
        return successful(response);
    }

    @PostMapping(value = "finishProcessSheet")
    @ApiOperation(value = "通知APMS完成生产单接口")
    public R finishProcessSheet(@Validated @RequestBody ApmsFinishProcessSheet apmsFinishProcessSheet) {
        checkToken();
        HashMap<String, Object> paramMap = new HashMap<>();
        paramMap.put("entity", apmsFinishProcessSheet);
        HttpResponse response = HttpUtil.createPost(ApiAddress + "/platform/api/finishProcessSheet")
                .header("token", Token).form(paramMap).execute();
        CommunicationLog communicationLog = new CommunicationLog();
        communicationLog.setCode("APMS" + System.currentTimeMillis());
        communicationLog.setLogName("WMS通知APMS完成生产单");
        communicationLog.setType(2);
        communicationLog.setContent("param:" + paramMap + "\nresult:" + response.body());
        setCommonField(communicationLog);
        communicationLogServiceBiz.save(communicationLog);
        return successful(response);
    }

    @PostMapping(value = "getApmsToken")
    @ApiOperation(value = "获取APMS Token")
    public R getApmsToken() {
        HashMap<String, Object> paramMap = new HashMap<>();
        paramMap.put("apiKey", "7ee8f59bbaeae27e");
        paramMap.put("platFormCode", "CUC");
        String result = HttpUtil.get(ApiAddress + "/platform/api/getToken", paramMap);
        JSON parse = JSONUtil.parse(result);
        Object token = parse.getByPath("token");
        String expireTime = parse.getByPath("expireTime").toString();
        Token = token.toString();
        TokenExpireTime = LocalDateTimeUtil.parse(expireTime);
        CommunicationLog communicationLog = new CommunicationLog();
        communicationLog.setCode("APMS" + System.currentTimeMillis());
        communicationLog.setLogName("WMS获取APMS Token");
        communicationLog.setType(2);
        communicationLog.setContent("param:" + paramMap + "\nresult:" + result);
        setCommonField(communicationLog);
        communicationLogServiceBiz.save(communicationLog);
        return successful(result);
    }

    @PostMapping(value = "getStoveCodeByWorkShopCode")
    @ApiOperation(value = "APMS通过车间编码获取炉号信息")
    public R getStoveCodeByWorkShopCode(String workShopCode) {
        checkToken();
        HashMap<String, Object> paramMap = new HashMap<>();
        paramMap.put("workShopCode", workShopCode);
        HttpResponse response = HttpUtil.createGet(ApiAddress + "/platform/api/getStoveCodeByWorkShopCode")
                .header("token", Token).form(paramMap).execute();
        System.out.println(response.body());
        CommunicationLog communicationLog = new CommunicationLog();
        communicationLog.setCode("APMS" + System.currentTimeMillis());
        communicationLog.setLogName("APMS通过车间编码获取炉号信息");
        communicationLog.setType(2);
        communicationLog.setContent("param:" + paramMap + "\nresult:" + response.body());
        setCommonField(communicationLog);
        communicationLogServiceBiz.save(communicationLog);
        return successful(response);
    }

    private void checkToken() {
        if (Token.isEmpty())
        {
            getApmsToken();
        }
        else if (TokenExpireTime.isBefore(LocalDateTime.now()))
        {
            getApmsToken();
        }
    }
}
